"use strict";
var chai = require("chai");
var chaiAsPromised = require("chai-as-promised");
var simple_mock_1 = require("simple-mock");
var mqtt = require("mqtt");
var mqtt_pubsub_1 = require("../mqtt-pubsub");
chai.use(chaiAsPromised);
var expect = chai.expect;
var listener;
var publishSpy = simple_mock_1.spy(function (channel, message, options, cb) { return listener && listener(channel, message); });
var subscribeSpy = simple_mock_1.spy(function (topic, options, cb) { return cb && cb(null, { topic: topic }); });
var unsubscribeSpy = simple_mock_1.spy(function (channel, options, cb) { return cb && cb(channel); });
var mqttPackage = mqtt;
var connect = function () {
    return {
        publish: publishSpy,
        subscribe: subscribeSpy,
        unsubscribe: unsubscribeSpy,
        on: function (event, cb) {
            if (event === 'message') {
                listener = cb;
            }
        },
    };
};
mqttPackage['connect'] = connect;
describe('MQTTPubSub', function () {
    var pubSub = new mqtt_pubsub_1.MQTTPubSub();
    it('can subscribe to specific mqtt channel and called when a message is published on it', function (done) {
        pubSub.subscribe('Posts', function (message) {
            try {
                expect(message).to.equals('test');
                done();
            }
            catch (e) {
                done(e);
            }
        }).then(function (subId) {
            expect(subId).to.be.a('number');
            pubSub.publish('Posts', 'test');
            pubSub.unsubscribe(subId);
        }).catch(function (err) { return done(err); });
    });
    it('can unsubscribe from specific mqtt channel', function (done) {
        pubSub.subscribe('Posts', function () { return null; }).then(function (subId) {
            pubSub.unsubscribe(subId);
            try {
                expect(unsubscribeSpy.callCount).to.equals(1);
                var call = unsubscribeSpy.lastCall;
                expect(call.args).to.have.members(['Posts']);
                done();
            }
            catch (e) {
                done(e);
            }
        });
    });
    it('cleans up correctly the memory when unsubscribing', function (done) {
        Promise.all([
            pubSub.subscribe('Posts', function () { return null; }),
            pubSub.subscribe('Posts', function () { return null; }),
        ])
            .then(function (_a) {
            var subId = _a[0], secondSubId = _a[1];
            try {
                expect(pubSub.subscriptionMap[subId]).not.to.be.an('undefined');
                pubSub.unsubscribe(subId);
                expect(pubSub.subscriptionMap[subId]).to.be.an('undefined');
                expect(function () { return pubSub.unsubscribe(subId); }).to.throw("There is no subscription of id \"" + subId + "\"");
                pubSub.unsubscribe(secondSubId);
                done();
            }
            catch (e) {
                done(e);
            }
        });
    });
    it('will not unsubscribe from the mqtt channel if there is another subscriber on it\'s subscriber list', function (done) {
        var subscriptionPromises = [
            pubSub.subscribe('Posts', function () {
                done('Not supposed to be triggered');
            }),
            pubSub.subscribe('Posts', function (msg) {
                try {
                    expect(msg).to.equals('test');
                    done();
                }
                catch (e) {
                    done(e);
                }
            }),
        ];
        Promise.all(subscriptionPromises).then(function (subIds) {
            try {
                expect(subIds.length).to.equals(2);
                pubSub.unsubscribe(subIds[0]);
                expect(unsubscribeSpy.callCount).to.equals(0);
                pubSub.publish('Posts', 'test');
                pubSub.unsubscribe(subIds[1]);
                expect(unsubscribeSpy.callCount).to.equals(1);
            }
            catch (e) {
                done(e);
            }
        });
    });
    it('will subscribe to mqtt channel only once', function (done) {
        var onMessage = function () { return null; };
        pubSub.subscribe('Posts', onMessage).then(function (id1) {
            return pubSub.subscribe('Posts', onMessage)
                .then(function (id2) { return [id1, id2]; });
        }).then(function (subIds) {
            try {
                expect(subIds.length).to.equals(2);
                expect(subscribeSpy.callCount).to.equals(1);
                pubSub.unsubscribe(subIds[0]);
                pubSub.unsubscribe(subIds[1]);
                done();
            }
            catch (e) {
                done(e);
            }
        });
    });
    it('can have multiple subscribers and all will be called when a message is published to this channel', function (done) {
        var onMessageSpy = simple_mock_1.spy(function () { return null; });
        var subscriptionPromises = [
            pubSub.subscribe('Posts', onMessageSpy),
            pubSub.subscribe('Posts', onMessageSpy),
        ];
        Promise.all(subscriptionPromises).then(function (subIds) {
            try {
                expect(subIds.length).to.equals(2);
                pubSub.publish('Posts', 'test');
                expect(onMessageSpy.callCount).to.equals(2);
                onMessageSpy.calls.forEach(function (call) {
                    expect(call.args).to.have.members(['test']);
                });
                pubSub.unsubscribe(subIds[0]);
                pubSub.unsubscribe(subIds[1]);
                done();
            }
            catch (e) {
                done(e);
            }
        });
    });
    it('can publish objects as well', function (done) {
        pubSub.subscribe('Posts', function (message) {
            try {
                expect(message).to.have.property('comment', 'This is amazing');
                done();
            }
            catch (e) {
                done(e);
            }
        }).then(function (subId) {
            try {
                pubSub.publish('Posts', { comment: 'This is amazing' });
                pubSub.unsubscribe(subId);
            }
            catch (e) {
                done(e);
            }
        });
    });
    it('throws if you try to unsubscribe with an unknown id', function () {
        return expect(function () { return pubSub.unsubscribe(123); })
            .to.throw('There is no subscription of id "123"');
    });
    it('can use transform function to convert the trigger name given into more explicit channel name', function (done) {
        var triggerTransform = function (trigger, _a) {
            var repoName = _a.repoName;
            return trigger + "." + repoName;
        };
        var pubsub = new mqtt_pubsub_1.MQTTPubSub({
            triggerTransform: triggerTransform,
        });
        var validateMessage = function (message) {
            try {
                expect(message).to.equals('test');
                done();
            }
            catch (e) {
                done(e);
            }
        };
        pubsub.subscribe('comments', validateMessage, { repoName: 'graphql-mqtt-subscriptions' }).then(function (subId) {
            pubsub.publish('comments.graphql-mqtt-subscriptions', 'test');
            pubsub.unsubscribe(subId);
        });
    });
    afterEach('Reset spy count', function () {
        publishSpy.reset();
        subscribeSpy.reset();
        unsubscribeSpy.reset();
    });
    after('Restore mqtt client', function () {
        simple_mock_1.restore();
    });
});
//# sourceMappingURL=tests.js.map